@@ -44,10 +44,12 @@ G_BEGIN_DECLS
 #define NM_VPN_PLUGIN_INFO_KF_GROUP_LIBNM        "libnm"
 #define NM_VPN_PLUGIN_INFO_KF_GROUP_GNOME        "GNOME"
 
+struct _NMVpnPluginInfo;
+
 /**
  * NMVpnPluginInfo:
  */
-typedef struct {
+typedef struct _NMVpnPluginInfo {
 	NM_AVAILABLE_IN_1_2
 	GObject parent;
 } NMVpnPluginInfo;
@@ -73,16 +75,28 @@ NMVpnPluginInfo *nm_vpn_plugin_info_new_with_data (const char *filename,
                                                    GKeyFile *keyfile,
                                                    GError **error);
 
+NM_AVAILABLE_IN_1_4
+NMVpnPluginInfo *nm_vpn_plugin_info_new_search_file (const char *name,
+                                                     const char *service);
+
 NM_AVAILABLE_IN_1_2
 const char *nm_vpn_plugin_info_get_name        (NMVpnPluginInfo *self);
 NM_AVAILABLE_IN_1_2
 const char *nm_vpn_plugin_info_get_filename    (NMVpnPluginInfo *self);
+NM_AVAILABLE_IN_1_4
+const char *nm_vpn_plugin_info_get_service     (NMVpnPluginInfo *self);
 NM_AVAILABLE_IN_1_2
 const char *nm_vpn_plugin_info_get_plugin      (NMVpnPluginInfo *self);
 NM_AVAILABLE_IN_1_2
 const char *nm_vpn_plugin_info_get_program     (NMVpnPluginInfo *self);
+NM_AVAILABLE_IN_1_4
+const char *nm_vpn_plugin_info_get_auth_dialog (NMVpnPluginInfo *self);
+NM_AVAILABLE_IN_1_4
+gboolean nm_vpn_plugin_info_supports_hints     (NMVpnPluginInfo *self);
 NM_AVAILABLE_IN_1_2
 gboolean nm_vpn_plugin_info_supports_multiple  (NMVpnPluginInfo *self);
+NM_AVAILABLE_IN_1_4
+const char *const*nm_vpn_plugin_info_get_aliases (NMVpnPluginInfo *self);
 NM_AVAILABLE_IN_1_2
 const char *nm_vpn_plugin_info_lookup_property (NMVpnPluginInfo *self, const char *group, const char *key);
 
@@ -102,6 +116,12 @@ NMVpnPluginInfo *nm_vpn_plugin_info_list_find_by_filename (GSList *list, const c
 NM_AVAILABLE_IN_1_2
 NMVpnPluginInfo *nm_vpn_plugin_info_list_find_by_service  (GSList *list, const char *service);
 
+NM_AVAILABLE_IN_1_4
+char *nm_vpn_plugin_info_list_find_service_type (GSList *list, const char *name);
+NM_AVAILABLE_IN_1_4
+char **nm_vpn_plugin_info_list_get_service_types (GSList *list,
+                                                  gboolean only_existing,
+                                                  gboolean with_abbreviations);
 
 NM_AVAILABLE_IN_1_2
 NMVpnEditorPlugin *nm_vpn_plugin_info_get_editor_plugin  (NMVpnPluginInfo *self);
