@@ -26,14 +26,12 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
-#include "nm-lldp-listener.h"
-#include "nm-sd.h"
+#include "devices/nm-lldp-listener.h"
+#include "systemd/nm-sd.h"
 
-#include "sd-lldp.h"
+#include "platform/tests/test-common.h"
 
-#include "test-common.h"
-
-#include "nm-test-utils.h"
+#include "nm-test-utils-core.h"
 
 /*****************************************************************************/
 
@@ -352,7 +350,7 @@ _test_recv_fixture_setup (TestRecvFixture *fixture, gconstpointer user_data)
 	struct ifreq ifr = { };
 	int fd, s;
 
-	fd = open ("/dev/net/tun", O_RDWR);
+	fd = open ("/dev/net/tun", O_RDWR | O_CLOEXEC);
 	g_assert (fd >= 0);
 
 	ifr.ifr_flags = IFF_TAP | IFF_NO_PI;
@@ -360,7 +358,7 @@ _test_recv_fixture_setup (TestRecvFixture *fixture, gconstpointer user_data)
 	g_assert (ioctl (fd, TUNSETIFF, &ifr) >= 0);
 
 	/* Bring the interface up */
-	s = socket (AF_INET, SOCK_DGRAM, 0);
+	s = socket (AF_INET, SOCK_DGRAM | SOCK_CLOEXEC, 0);
 	g_assert (s >= 0);
 	ifr.ifr_flags |= IFF_UP;
 	g_assert (ioctl (s, SIOCSIFFLAGS, &ifr) >= 0);
@@ -434,6 +432,8 @@ _test_recv_fixture_teardown (TestRecvFixture *fixture, gconstpointer user_data)
 
 /*****************************************************************************/
 
+NMTstpSetupFunc const _nmtstp_setup_platform_func = nm_linux_platform_setup;
+
 void
 _nmtstp_init_tests (int *argc, char ***argv)
 {
