@@ -22,11 +22,8 @@
 #ifndef __NETWORKMANAGER_MODEM_H__
 #define __NETWORKMANAGER_MODEM_H__
 
-#include "nm-default.h"
-#include "ppp-manager/nm-ppp-manager.h"
-#include "nm-device.h"
-
-G_BEGIN_DECLS
+#include "ppp/nm-ppp-manager.h"
+#include "devices/nm-device.h"
 
 #define NM_TYPE_MODEM            (nm_modem_get_type ())
 #define NM_MODEM(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), NM_TYPE_MODEM, NMModem))
@@ -79,7 +76,7 @@ typedef enum {
  * combination of flags is possible.  For example, (%NM_MODEM_IP_TYPE_IPV4 |
  * %NM_MODEM_IP_TYPE_IPV6) indicates that the modem supports IPv4 and IPv6
  * but not simultaneously on the same bearer.
- */ 
+ */
 typedef enum {
 	NM_MODEM_IP_TYPE_UNKNOWN = 0x0,
 	NM_MODEM_IP_TYPE_IPV4 = 0x1,
@@ -101,11 +98,16 @@ typedef enum {  /*< underscore_name=nm_modem_state >*/
 	NM_MODEM_STATE_DISCONNECTING = 10,
 	NM_MODEM_STATE_CONNECTING    = 11,
 	NM_MODEM_STATE_CONNECTED     = 12,
+
+	_NM_MODEM_STATE_LAST0,
+	_NM_MODEM_STATE_LAST = _NM_MODEM_STATE_LAST0 -1,
 } NMModemState;
 
+struct _NMModemPrivate;
 
 typedef struct {
 	GObject parent;
+	struct _NMModemPrivate *_priv;
 } NMModem;
 
 typedef struct {
@@ -156,26 +158,6 @@ typedef struct {
 	void     (*deactivate_cleanup)             (NMModem *self, NMDevice *device);
 
 	gboolean (*owns_port)                      (NMModem *self, const char *iface);
-
-	/* Signals */
-	void (*ppp_stats)  (NMModem *self, guint32 in_bytes, guint32 out_bytes);
-	void (*ppp_failed) (NMModem *self, NMDeviceStateReason reason);
-
-	void (*prepare_result)    (NMModem *self, gboolean success, NMDeviceStateReason reason);
-	void (*ip4_config_result) (NMModem *self, NMIP4Config *config, GError *error);
-	void (*ip6_config_result) (NMModem *self,
-	                           NMIP6Config *config,
-	                           gboolean do_slaac,
-	                           GError *error);
-
-	void (*auth_requested)    (NMModem *self);
-	void (*auth_result)       (NMModem *self, GError *error);
-
-	void (*state_changed)     (NMModem *self,
-	                           NMModemState new_state,
-	                           NMModemState old_state);
-
-	void (*removed)           (NMModem *self);
 } NMModemClass;
 
 GType nm_modem_get_type (void);
@@ -268,6 +250,4 @@ void nm_modem_emit_ip6_config_result (NMModem *self,
 
 const gchar *nm_modem_ip_type_to_string (NMModemIPType ip_type);
 
-G_END_DECLS
-
 #endif /* __NETWORKMANAGER_MODEM_H__ */
