@@ -21,16 +21,23 @@
 
 #include "nm-default.h"
 
+#include "nm-default-route-manager.h"
+
 #include <string.h>
 
-#include "nm-default-route-manager.h"
-#include "nm-device.h"
-#include "nm-vpn-connection.h"
-#include "nm-platform.h"
+#include "devices/nm-device.h"
+#include "vpn/nm-vpn-connection.h"
+#include "platform/nm-platform.h"
 #include "nm-manager.h"
 #include "nm-ip4-config.h"
 #include "nm-ip6-config.h"
-#include "nm-activation-request.h"
+#include "nm-act-request.h"
+
+/*****************************************************************************/
+
+NM_GOBJECT_PROPERTIES_DEFINE_BASE (
+	PROP_PLATFORM,
+);
 
 typedef struct {
 	GPtrArray *entries_ip4;
@@ -54,16 +61,23 @@ typedef struct {
 	NMPlatform *platform;
 } NMDefaultRouteManagerPrivate;
 
-#define NM_DEFAULT_ROUTE_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), NM_TYPE_DEFAULT_ROUTE_MANAGER, NMDefaultRouteManagerPrivate))
+struct _NMDefaultRouteManager {
+	GObject parent;
+	NMDefaultRouteManagerPrivate _priv;
+};
+
+struct _NMDefaultRouteManagerClass {
+	GObjectClass parent;
+};
 
 G_DEFINE_TYPE (NMDefaultRouteManager, nm_default_route_manager, G_TYPE_OBJECT)
 
-NM_GOBJECT_PROPERTIES_DEFINE_BASE (
-	PROP_PLATFORM,
-);
+#define NM_DEFAULT_ROUTE_MANAGER_GET_PRIVATE(self) _NM_GET_PRIVATE (self, NMDefaultRouteManager, NM_IS_DEFAULT_ROUTE_MANAGER)
 
 NM_DEFINE_SINGLETON_GETTER (NMDefaultRouteManager, nm_default_route_manager_get, NM_TYPE_DEFAULT_ROUTE_MANAGER);
 
+/*****************************************************************************/
+
 #define _NMLOG_PREFIX_NAME   "default-route"
 #undef  _NMLOG_ENABLED
 #define _NMLOG_ENABLED(level, addr_family) \
@@ -127,11 +141,11 @@ NM_DEFINE_SINGLETON_GETTER (NMDefaultRouteManager, nm_default_route_manager_get,
         } \
     } G_STMT_END
 
-/***********************************************************************************/
+/*****************************************************************************/
 
 static void _resync_idle_cancel (NMDefaultRouteManager *self);
 
-/***********************************************************************************/
+/*****************************************************************************/
 
 typedef struct {
 	union {
@@ -200,7 +214,7 @@ _vt_routes_has_entry (const VTableIP *vtable, GArray *routes, const Entry *entry
 		for (i = 0; i < routes->len; i++) {
 			NMPlatformIP4Route *r = &g_array_index (routes, NMPlatformIP4Route, i);
 
-			route.rx.source = r->source;
+			route.rx.rt_source = r->rt_source;
 			if (nm_platform_ip4_route_cmp (r, &route.r4) == 0)
 				return TRUE;
 		}
@@ -208,7 +222,7 @@ _vt_routes_has_entry (const VTableIP *vtable, GArray *routes, const Entry *entry
 		for (i = 0; i < routes->len; i++) {
 			NMPlatformIP6Route *r = &g_array_index (routes, NMPlatformIP6Route, i);
 
-			route.rx.source = r->source;
+			route.rx.rt_source = r->rt_source;
 			if (nm_platform_ip6_route_cmp (r, &route.r6) == 0)
 				return TRUE;
 		}
@@ -289,7 +303,7 @@ _platform_route_sync_add (const VTableIP *vtable, NMDefaultRouteManager *self, g
 	if (vtable->vt->is_ip4) {
 		success = nm_platform_ip4_route_add (priv->platform,
 		                                     entry->route.rx.ifindex,
-		                                     entry->route.rx.source,
+		                                     entry->route.rx.rt_source,
 		                                     0,
 		                                     0,
 		                                     entry->route.r4.gateway,
@@ -299,7 +313,7 @@ _platform_route_sync_add (const VTableIP *vtable, NMDefaultRouteManager *self, g
 	} else {
 		success = nm_platform_ip6_route_add (priv->platform,
 		                                     entry->route.rx.ifindex,
-		                                     entry->route.rx.source,
+		                                     entry->route.rx.rt_source,
 		                                     in6addr_any,
 		                                     0,
 		                                     entry->route.r6.gateway,
@@ -695,7 +709,7 @@ _entry_at_idx_remove (const VTableIP *vtable, NMDefaultRouteManager *self, guint
 	_entry_free (entry);
 }
 
-/***********************************************************************************/
+/*****************************************************************************/
 
 static void
 _ipx_update_default_route (const VTableIP *vtable, NMDefaultRouteManager *self, gpointer source)
@@ -765,7 +779,7 @@ _ipx_update_default_route (const VTableIP *vtable, NMDefaultRouteManager *self,
 				 * the device. */
 				memset (&rt, 0, sizeof (rt));
 				rt.rx.ifindex = ip_ifindex;
-				rt.rx.source = NM_IP_CONFIG_SOURCE_UNKNOWN;
+				rt.rx.rt_source = NM_IP_CONFIG_SOURCE_UNKNOWN;
 				rt.rx.metric = G_MAXUINT32;
 				default_route = &rt.rx;
 
@@ -787,7 +801,7 @@ _ipx_update_default_route (const VTableIP *vtable, NMDefaultRouteManager *self,
 					if (vpn_config) {
 						never_default = nm_ip4_config_get_never_default (vpn_config);
 						rt.r4.ifindex = ip_ifindex;
-						rt.r4.source = NM_IP_CONFIG_SOURCE_VPN;
+						rt.r4.rt_source = NM_IP_CONFIG_SOURCE_VPN;
 						rt.r4.gateway = nm_ip4_config_get_gateway (vpn_config);
 						rt.r4.metric = nm_vpn_connection_get_ip4_route_metric (vpn);
 						rt.r4.mss = nm_ip4_config_get_mss (vpn_config);
@@ -802,7 +816,7 @@ _ipx_update_default_route (const VTableIP *vtable, NMDefaultRouteManager *self,
 
 						never_default = nm_ip6_config_get_never_default (vpn_config);
 						rt.r6.ifindex = ip_ifindex;
-						rt.r6.source = NM_IP_CONFIG_SOURCE_VPN;
+						rt.r6.rt_source = NM_IP_CONFIG_SOURCE_VPN;
 						rt.r6.gateway = int_gw ? *int_gw : in6addr_any;
 						rt.r6.metric = nm_vpn_connection_get_ip6_route_metric (vpn);
 						rt.r6.mss = nm_ip6_config_get_mss (vpn_config);
@@ -888,7 +902,7 @@ nm_default_route_manager_ip6_update_default_route (NMDefaultRouteManager *self,
 	_ipx_update_default_route (&vtable_ip6, self, source);
 }
 
-/***********************************************************************************/
+/*****************************************************************************/
 
 static gboolean
 _ipx_connection_has_default_route (const VTableIP *vtable, NMDefaultRouteManager *self, NMConnection *connection, gboolean *out_is_never_default)
@@ -947,7 +961,7 @@ nm_default_route_manager_ip6_connection_has_default_route (NMDefaultRouteManager
 	return _ipx_connection_has_default_route (&vtable_ip6, self, connection, out_is_never_default);
 }
 
-/***********************************************************************************/
+/*****************************************************************************/
 
 static NMDevice *
 _ipx_get_best_device (const VTableIP *vtable, NMDefaultRouteManager *self, const GSList *devices)
@@ -1085,7 +1099,7 @@ nm_default_route_manager_ip6_get_best_device (NMDefaultRouteManager *self, const
 		return _ipx_get_best_activating_device (&vtable_ip6, self, devices, preferred_device);
 }
 
-/***********************************************************************************/
+/*****************************************************************************/
 
 static gpointer
 _ipx_get_best_config (const VTableIP *vtable,
@@ -1218,7 +1232,7 @@ nm_default_route_manager_ip6_get_best_config (NMDefaultRouteManager *self,
 	                             out_vpn);
 }
 
-/***********************************************************************************/
+/*****************************************************************************/
 
 static GPtrArray *
 _v4_get_entries (NMDefaultRouteManagerPrivate *priv)
@@ -1242,7 +1256,7 @@ static const VTableIP vtable_ip6 = {
 	.get_entries                    = _v6_get_entries,
 };
 
-/***********************************************************************************/
+/*****************************************************************************/
 
 static gboolean
 _resync_idle_now (NMDefaultRouteManager *self)
@@ -1350,12 +1364,14 @@ _platform_ipx_route_changed_cb (const VTableIP *vtable,
 
 static void
 _platform_changed_cb (NMPlatform *platform,
-                      NMPObjectType obj_type,
+                      int obj_type_i,
                       int ifindex,
                       gpointer platform_object,
-                      NMPlatformSignalChangeType change_type,
+                      int change_type_i,
                       NMDefaultRouteManager *self)
 {
+	const NMPObjectType obj_type = obj_type_i;
+
 	switch (obj_type) {
 	case NMP_OBJECT_TYPE_IP4_ADDRESS:
 		_platform_ipx_route_changed_cb (&vtable_ip4, self, NULL);
@@ -1374,7 +1390,7 @@ _platform_changed_cb (NMPlatform *platform,
 	}
 }
 
-/***********************************************************************************/
+/*****************************************************************************/
 
 static void
 set_property (GObject *object, guint prop_id,
@@ -1397,6 +1413,8 @@ set_property (GObject *object, guint prop_id,
 	}
 }
 
+/*****************************************************************************/
+
 static void
 nm_default_route_manager_init (NMDefaultRouteManager *self)
 {
@@ -1464,9 +1482,6 @@ nm_default_route_manager_class_init (NMDefaultRouteManagerClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	g_type_class_add_private (klass, sizeof (NMDefaultRouteManagerPrivate));
-
-	/* virtual methods */
 	object_class->constructed = constructed;
 	object_class->dispose = dispose;
 	object_class->set_property = set_property;
@@ -1477,7 +1492,6 @@ nm_default_route_manager_class_init (NMDefaultRouteManagerClass *klass)
 	                         G_PARAM_WRITABLE |
 	                         G_PARAM_CONSTRUCT_ONLY |
 	                         G_PARAM_STATIC_STRINGS);
+
 	g_object_class_install_properties (object_class, _PROPERTY_ENUMS_LAST, obj_properties);
-
 }
-
