@@ -33,18 +33,7 @@
 #define NM_IS_IP6_CONFIG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), NM_TYPE_IP6_CONFIG))
 #define NM_IP6_CONFIG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), NM_TYPE_IP6_CONFIG, NMIP6ConfigClass))
 
-struct _NMIP6ConfigPrivate;
-
-struct _NMIP6Config {
-	NMExportedObject parent;
-
-	/* private */
-	struct _NMIP6ConfigPrivate *priv;
-};
-
-typedef struct {
-	NMExportedObjectClass parent;
-} NMIP6ConfigClass;
+typedef struct _NMIP6ConfigClass NMIP6ConfigClass;
 
 /* internal */
 #define NM_IP6_CONFIG_IFINDEX "ifindex"
@@ -71,13 +60,13 @@ NMIP6Config * nm_ip6_config_new_cloned (const NMIP6Config *src);
 
 int nm_ip6_config_get_ifindex (const NMIP6Config *config);
 
-/* Integration with nm-platform and nm-setting */
+
 NMIP6Config *nm_ip6_config_capture (int ifindex, gboolean capture_resolv_conf, NMSettingIP6ConfigPrivacy use_temporary);
 gboolean nm_ip6_config_commit (const NMIP6Config *config, int ifindex, gboolean routes_full_sync);
 void nm_ip6_config_merge_setting (NMIP6Config *config, NMSettingIPConfig *setting, guint32 default_route_metric);
 NMSetting *nm_ip6_config_create_setting (const NMIP6Config *config);
 
-/* Utility functions */
+
 void nm_ip6_config_merge (NMIP6Config *dst, const NMIP6Config *src, NMIPConfigMergeFlags merge_flags);
 void nm_ip6_config_subtract (NMIP6Config *dst, const NMIP6Config *src);
 void nm_ip6_config_intersect (NMIP6Config *dst, const NMIP6Config *src);
@@ -85,14 +74,13 @@ gboolean nm_ip6_config_replace (NMIP6Config *dst, const NMIP6Config *src, gboole
 int nm_ip6_config_destination_is_direct (const NMIP6Config *config, const struct in6_addr *dest, guint8 plen);
 void nm_ip6_config_dump (const NMIP6Config *config, const char *detail);
 
-/* Gateways */
+
 void nm_ip6_config_set_never_default (NMIP6Config *config, gboolean never_default);
 gboolean nm_ip6_config_get_never_default (const NMIP6Config *config);
 void nm_ip6_config_set_gateway (NMIP6Config *config, const struct in6_addr *);
 const struct in6_addr *nm_ip6_config_get_gateway (const NMIP6Config *config);
 gint64 nm_ip6_config_get_route_metric (const NMIP6Config *config);
 
-/* Addresses */
 void nm_ip6_config_reset_addresses (NMIP6Config *config);
 void nm_ip6_config_add_address (NMIP6Config *config, const NMPlatformIP6Address *address);
 void nm_ip6_config_del_address (NMIP6Config *config, guint i);
@@ -100,58 +88,55 @@ guint nm_ip6_config_get_num_addresses (const NMIP6Config *config);
 const NMPlatformIP6Address *nm_ip6_config_get_address (const NMIP6Config *config, guint i);
 const NMPlatformIP6Address *nm_ip6_config_get_address_first_nontentative (const NMIP6Config *config, gboolean linklocal);
 gboolean nm_ip6_config_address_exists (const NMIP6Config *config, const NMPlatformIP6Address *address);
-gboolean nm_ip6_config_addresses_sort (NMIP6Config *config, NMSettingIP6ConfigPrivacy use_temporary);
+gboolean nm_ip6_config_addresses_sort (NMIP6Config *config);
+gboolean nm_ip6_config_has_any_dad_pending (const NMIP6Config *self,
+                                            const NMIP6Config *candidates);
 
-/* Routes */
 void nm_ip6_config_reset_routes (NMIP6Config *config);
 void nm_ip6_config_add_route (NMIP6Config *config, const NMPlatformIP6Route *route);
 void nm_ip6_config_del_route (NMIP6Config *config, guint i);
-guint32 nm_ip6_config_get_num_routes (const NMIP6Config *config);
-const NMPlatformIP6Route *nm_ip6_config_get_route (const NMIP6Config *config, guint32 i);
+guint nm_ip6_config_get_num_routes (const NMIP6Config *config);
+const NMPlatformIP6Route *nm_ip6_config_get_route (const NMIP6Config *config, guint i);
 
 const NMPlatformIP6Route *nm_ip6_config_get_direct_route_for_host (const NMIP6Config *config, const struct in6_addr *host);
 const NMPlatformIP6Address *nm_ip6_config_get_subnet_for_host (const NMIP6Config *config, const struct in6_addr *host);
 
-/* Nameservers */
 void nm_ip6_config_reset_nameservers (NMIP6Config *config);
 void nm_ip6_config_add_nameserver (NMIP6Config *config, const struct in6_addr *nameserver);
 void nm_ip6_config_del_nameserver (NMIP6Config *config, guint i);
-guint32 nm_ip6_config_get_num_nameservers (const NMIP6Config *config);
+guint nm_ip6_config_get_num_nameservers (const NMIP6Config *config);
 const struct in6_addr *nm_ip6_config_get_nameserver (const NMIP6Config *config, guint i);
 
-/* Domains */
 void nm_ip6_config_reset_domains (NMIP6Config *config);
 void nm_ip6_config_add_domain (NMIP6Config *config, const char *domain);
 void nm_ip6_config_del_domain (NMIP6Config *config, guint i);
-guint32 nm_ip6_config_get_num_domains (const NMIP6Config *config);
+guint nm_ip6_config_get_num_domains (const NMIP6Config *config);
 const char * nm_ip6_config_get_domain (const NMIP6Config *config, guint i);
 
-/* Search lists */
 void nm_ip6_config_reset_searches (NMIP6Config *config);
 void nm_ip6_config_add_search (NMIP6Config *config, const char *search);
 void nm_ip6_config_del_search (NMIP6Config *config, guint i);
-guint32 nm_ip6_config_get_num_searches (const NMIP6Config *config);
+guint nm_ip6_config_get_num_searches (const NMIP6Config *config);
 const char * nm_ip6_config_get_search (const NMIP6Config *config, guint i);
 
-/* DNS options */
 void nm_ip6_config_reset_dns_options (NMIP6Config *config);
 void nm_ip6_config_add_dns_option (NMIP6Config *config, const char *option);
 void nm_ip6_config_del_dns_option (NMIP6Config *config, guint i);
-guint32 nm_ip6_config_get_num_dns_options (const NMIP6Config *config);
+guint nm_ip6_config_get_num_dns_options (const NMIP6Config *config);
 const char * nm_ip6_config_get_dns_option (const NMIP6Config *config, guint i);
 
-/* DNS priority */
 void nm_ip6_config_set_dns_priority (NMIP6Config *config, gint priority);
 gint nm_ip6_config_get_dns_priority (const NMIP6Config *config);
 
-/* MSS */
 void nm_ip6_config_set_mss (NMIP6Config *config, guint32 mss);
 guint32 nm_ip6_config_get_mss (const NMIP6Config *config);
 
 void nm_ip6_config_hash (const NMIP6Config *config, GChecksum *sum, gboolean dns_only);
 gboolean nm_ip6_config_equal (const NMIP6Config *a, const NMIP6Config *b);
 
-/******************************************************/
+void nm_ip6_config_set_privacy (NMIP6Config *config, NMSettingIP6ConfigPrivacy privacy);
+
+/*****************************************************************************/
 /* Testing-only functions */
 
 gboolean nm_ip6_config_capture_resolv_conf (GArray *nameservers,
