@@ -32,11 +32,6 @@
 #include <strings.h>
 #include <string.h>
 
-#if defined (NO_SYSTEMD_JOURNAL) && defined (SYSTEMD_JOURNAL)
-#undef SYSTEMD_JOURNAL
-#define SYSTEMD_JOURNAL 0
-#endif
-
 #if SYSTEMD_JOURNAL
 #define SD_JOURNAL_SUPPRESS_LOCATION
 #include <systemd/sd-journal.h>
@@ -45,37 +40,16 @@
 #include "nm-errors.h"
 #include "nm-core-utils.h"
 
-typedef enum {
-	LOG_FORMAT_FLAG_NONE                                = 0,
-	LOG_FORMAT_FLAG_TIMESTAMP_DEBUG                     = (1LL << 0),
-	LOG_FORMAT_FLAG_TIMESTAMP_INFO                      = (1LL << 1),
-	LOG_FORMAT_FLAG_TIMESTAMP_ERROR                     = (1LL << 2),
-	LOG_FORMAT_FLAG_LOCATION_DEBUG                      = (1LL << 3),
-	LOG_FORMAT_FLAG_LOCATION_INFO                       = (1LL << 4),
-	LOG_FORMAT_FLAG_LOCATION_ERROR                      = (1LL << 5),
-	LOG_FORMAT_FLAG_ALIGN_LOCATION                      = (1LL << 6),
-
-	_LOG_FORMAT_FLAG_TIMESTAMP                          = LOG_FORMAT_FLAG_TIMESTAMP_DEBUG |
-	                                                      LOG_FORMAT_FLAG_TIMESTAMP_INFO |
-	                                                      LOG_FORMAT_FLAG_TIMESTAMP_ERROR,
-	_LOG_FORMAT_FLAG_LOCATION                           = LOG_FORMAT_FLAG_LOCATION_DEBUG |
-	                                                      LOG_FORMAT_FLAG_LOCATION_INFO |
-	                                                      LOG_FORMAT_FLAG_LOCATION_ERROR,
-
-	_LOG_FORMAT_FLAG_LEVEL_DEBUG                        = LOG_FORMAT_FLAG_TIMESTAMP_DEBUG |
-	                                                      LOG_FORMAT_FLAG_LOCATION_DEBUG,
-	_LOG_FORMAT_FLAG_LEVEL_INFO                         = LOG_FORMAT_FLAG_TIMESTAMP_INFO |
-	                                                      LOG_FORMAT_FLAG_LOCATION_INFO,
-	_LOG_FORMAT_FLAG_LEVEL_ERROR                        = LOG_FORMAT_FLAG_TIMESTAMP_ERROR |
-	                                                      LOG_FORMAT_FLAG_LOCATION_ERROR,
-
-	_LOG_FORMAT_FLAG_SYSLOG                             = _LOG_FORMAT_FLAG_TIMESTAMP |
-	                                                      LOG_FORMAT_FLAG_LOCATION_DEBUG |
-	                                                      LOG_FORMAT_FLAG_LOCATION_ERROR |
-	                                                      LOG_FORMAT_FLAG_ALIGN_LOCATION,
-
-	_LOG_FORMAT_FLAG_DEFAULT                            = _LOG_FORMAT_FLAG_TIMESTAMP,
-} LogFormatFlags;
+/* often we have some static string where we need to know the maximum length.
+ * _MAX_LEN() returns @max but adds a debugging assertion that @str is indeed
+ * shorter then @mac. */
+#define _MAX_LEN(max, str) \
+	({ \
+		const char *const _str = (str); \
+		\
+		nm_assert (_str && strlen (str) < (max)); \
+		(max); \
+	})
 
 void (*_nm_logging_clear_platform_logging_cache) (void);
 
@@ -93,16 +67,35 @@ typedef struct {
 typedef struct {
 	const char *name;
 	const char *level_str;
+
+	/* nm-logging uses syslog internally. Note that the three most-verbose syslog levels
+	 * are LOG_DEBUG, LOG_INFO and LOG_NOTICE. Journal already highlights LOG_NOTICE
+	 * as special.
+	 *
+	 * On the other hand, we have three levels LOGL_TRACE, LOGL_DEBUG and LOGL_INFO,
+	 * which are regular messages not to be highlighted. For that reason, we must map
+	 * LOGL_TRACE and LOGL_DEBUG both to syslog level LOG_DEBUG. */
 	int syslog_level;
+
 	GLogLevelFlags g_log_level;
-	LogFormatFlags log_format_level;
 } LogLevelDesc;
 
-static struct {
+NMLogDomain _nm_logging_enabled_state[_LOGL_N_REAL] = {
+	/* nm_logging_setup ("INFO", LOGD_DEFAULT_STRING, NULL, NULL);
+	 *
+	 * Note: LOGD_VPN_PLUGIN is special and must be disabled for
+	 * DEBUG and TRACE levels. */
+	[LOGL_INFO] = LOGD_DEFAULT,
+	[LOGL_WARN] = LOGD_DEFAULT,
+	[LOGL_ERR]  = LOGD_DEFAULT,
+};
+
+static struct Global {
 	NMLogLevel log_level;
-	NMLogDomain logging[_LOGL_N_REAL];
-	gboolean logging_set_up;
-	LogFormatFlags log_format_flags;
+	bool uses_syslog:1;
+	bool syslog_identifier_initialized:1;
+	const char *prefix;
+	const char *syslog_identifier;
 	enum {
 		LOG_BACKEND_GLIB,
 		LOG_BACKEND_SYSLOG,
@@ -111,22 +104,24 @@ static struct {
 	char *logging_domains_to_string;
 	const LogLevelDesc level_desc[_LOGL_N];
 
-#define _DOMAIN_DESC_LEN 37
+#define _DOMAIN_DESC_LEN 39
 	/* Would be nice to use C99 flexible array member here,
 	 * but that feature doesn't seem well supported. */
 	const LogDesc domain_desc[_DOMAIN_DESC_LEN];
 } global = {
+	/* nm_logging_setup ("INFO", LOGD_DEFAULT_STRING, NULL, NULL); */
 	.log_level = LOGL_INFO,
 	.log_backend = LOG_BACKEND_GLIB,
-	.log_format_flags = _LOG_FORMAT_FLAG_DEFAULT,
+	.syslog_identifier = "SYSLOG_IDENTIFIER="G_LOG_DOMAIN,
+	.prefix = "",
 	.level_desc = {
-		[LOGL_TRACE] = { "TRACE", "<trace>", LOG_DEBUG,   G_LOG_LEVEL_DEBUG,   _LOG_FORMAT_FLAG_LEVEL_DEBUG },
-		[LOGL_DEBUG] = { "DEBUG", "<debug>", LOG_INFO,    G_LOG_LEVEL_DEBUG,   _LOG_FORMAT_FLAG_LEVEL_DEBUG },
-		[LOGL_INFO]  = { "INFO",  "<info>",  LOG_INFO,    G_LOG_LEVEL_INFO,    _LOG_FORMAT_FLAG_LEVEL_INFO },
-		[LOGL_WARN]  = { "WARN",  "<warn>",  LOG_WARNING, G_LOG_LEVEL_MESSAGE, _LOG_FORMAT_FLAG_LEVEL_INFO },
-		[LOGL_ERR]   = { "ERR",   "<error>", LOG_ERR,     G_LOG_LEVEL_MESSAGE, _LOG_FORMAT_FLAG_LEVEL_ERROR },
-		[_LOGL_OFF]  = { "OFF",   NULL,      0,           0,                   0 },
-		[_LOGL_KEEP] = { "KEEP",  NULL,      0,           0,                   0 },
+		[LOGL_TRACE] = { "TRACE", "<trace>", LOG_DEBUG,   G_LOG_LEVEL_DEBUG,   },
+		[LOGL_DEBUG] = { "DEBUG", "<debug>", LOG_DEBUG,   G_LOG_LEVEL_DEBUG,   },
+		[LOGL_INFO]  = { "INFO",  "<info>",  LOG_INFO,    G_LOG_LEVEL_INFO,    },
+		[LOGL_WARN]  = { "WARN",  "<warn>",  LOG_WARNING, G_LOG_LEVEL_MESSAGE, },
+		[LOGL_ERR]   = { "ERR",   "<error>", LOG_ERR,     G_LOG_LEVEL_MESSAGE, },
+		[_LOGL_OFF]  = { "OFF",   NULL,      0,           0,                   },
+		[_LOGL_KEEP] = { "KEEP",  NULL,      0,           0,                   },
 	},
 	.domain_desc = {
 		{ LOGD_PLATFORM,  "PLATFORM" },
@@ -165,6 +160,8 @@ static struct {
 		{ LOGD_DISPATCH,  "DISPATCH" },
 		{ LOGD_AUDIT,     "AUDIT" },
 		{ LOGD_SYSTEMD,   "SYSTEMD" },
+		{ LOGD_VPN_PLUGIN,"VPN_PLUGIN" },
+		{ LOGD_PROXY,     "PROXY" },
 		{ 0, NULL }
 		/* keep _DOMAIN_DESC_LEN in sync */
 	},
@@ -179,25 +176,78 @@ G_STATIC_ASSERT (sizeof (NMLogDomain) >= sizeof (guint64));
 #define LOGD_DHCP_STRING    "DHCP"
 #define LOGD_IP_STRING      "IP"
 
-/************************************************************************/
+/*****************************************************************************/
 
 static char *_domains_to_string (gboolean include_level_override);
 
-/************************************************************************/
+/*****************************************************************************/
 
-static void
-_ensure_initialized (void)
+static gboolean
+_syslog_identifier_valid_domain (const char *domain)
 {
-	if (G_UNLIKELY (!global.logging_set_up)) {
-		int errsv = errno;
+	char c;
 
-		nm_logging_setup ("INFO", LOGD_DEFAULT_STRING, NULL, NULL);
+	if (!domain || !domain[0])
+		return FALSE;
 
-		/* must ensure that errno is not modified. */
-		errno = errsv;
+	/* we pass the syslog identifier as format string. No funny stuff. */
+
+	for (; (c = domain[0]); domain++) {
+		if (   (c >= 'a' && c <= 'z')
+		    || (c >= 'A' && c <= 'Z')
+		    || (c >= '0' && c <= '9')
+		    || NM_IN_SET (c, '-', '_'))
+			continue;
+		return FALSE;
 	}
+	return TRUE;
 }
 
+static gboolean
+_syslog_identifier_assert (const struct Global *gl)
+{
+	g_assert (gl);
+	g_assert (gl->syslog_identifier);
+	g_assert (g_str_has_prefix (gl->syslog_identifier, "SYSLOG_IDENTIFIER="));
+	g_assert (_syslog_identifier_valid_domain (&gl->syslog_identifier[NM_STRLEN ("SYSLOG_IDENTIFIER=")]));
+	return TRUE;
+}
+
+static const char *
+syslog_identifier_domain (const struct Global *gl)
+{
+	nm_assert (_syslog_identifier_assert (gl));
+	return &gl->syslog_identifier[NM_STRLEN ("SYSLOG_IDENTIFIER=")];
+}
+
+#if SYSTEMD_JOURNAL
+static const char *
+syslog_identifier_full (const struct Global *gl)
+{
+	nm_assert (_syslog_identifier_assert (gl));
+	return &gl->syslog_identifier[0];
+}
+#endif
+
+void
+nm_logging_set_syslog_identifier (const char *domain)
+{
+	if (global.log_backend != LOG_BACKEND_GLIB)
+		g_return_if_reached ();
+
+	if (!_syslog_identifier_valid_domain (domain))
+		g_return_if_reached ();
+
+	if (global.syslog_identifier_initialized)
+		g_return_if_reached ();
+
+	global.syslog_identifier_initialized = TRUE;
+	global.syslog_identifier = g_strdup_printf ("SYSLOG_IDENTIFIER=%s", domain);
+	nm_assert (_syslog_identifier_assert (&global));
+}
+
+/*****************************************************************************/
+
 static gboolean
 match_log_level (const char  *level,
                  NMLogLevel  *out_level,
@@ -224,7 +274,7 @@ nm_logging_setup (const char  *level,
                   GError     **error)
 {
 	GString *unrecognized = NULL;
-	NMLogDomain new_logging[G_N_ELEMENTS (global.logging)];
+	NMLogDomain new_logging[G_N_ELEMENTS (_nm_logging_enabled_state)];
 	NMLogLevel new_log_level = global.log_level;
 	char **tmp, **iter;
 	int i;
@@ -235,13 +285,8 @@ nm_logging_setup (const char  *level,
 	g_return_val_if_fail (!error || !*error, FALSE);
 
 	/* domains */
-	if (!domains || !*domains) {
-		domains = global.logging_set_up
-		          ? (domains_free = _domains_to_string (FALSE))
-		          : LOGD_DEFAULT_STRING;
-	}
-
-	global.logging_set_up = TRUE;
+	if (!domains || !*domains)
+		domains = (domains_free = _domains_to_string (FALSE));
 
 	for (i = 0; i < G_N_ELEMENTS (new_logging); i++)
 		new_logging[i] = 0;
@@ -253,7 +298,7 @@ nm_logging_setup (const char  *level,
 		if (new_log_level == _LOGL_KEEP) {
 			new_log_level = global.log_level;
 			for (i = 0; i < G_N_ELEMENTS (new_logging); i++)
-				new_logging[i] = global.logging[i];
+				new_logging[i] = _nm_logging_enabled_state[i];
 		}
 	}
 
@@ -264,6 +309,11 @@ nm_logging_setup (const char  *level,
 		NMLogDomain bits;
 		char *p;
 
+		/* LOGD_VPN_PLUGIN is protected, that is, when setting ALL or DEFAULT,
+		 * it does not enable the verbose levels DEBUG and TRACE, because that
+		 * may expose sensitive data. */
+		NMLogDomain protect = LOGD_NONE;
+
 		if (!strlen (*iter))
 			continue;
 
@@ -279,12 +329,21 @@ nm_logging_setup (const char  *level,
 
 		bits = 0;
 
+		if (domains_free) {
+			/* The caller didn't provide any domains to set (`nmcli general logging level DEBUG`).
+			 * We reset all domains that were previously set, but we still want to protect
+			 * VPN_PLUGIN domain. */
+			protect = LOGD_VPN_PLUGIN;
+		}
+
 		/* Check for combined domains */
-		if (!g_ascii_strcasecmp (*iter, LOGD_ALL_STRING))
+		if (!g_ascii_strcasecmp (*iter, LOGD_ALL_STRING)) {
 			bits = LOGD_ALL;
-		else if (!g_ascii_strcasecmp (*iter, LOGD_DEFAULT_STRING))
+			protect = LOGD_VPN_PLUGIN;
+		} else if (!g_ascii_strcasecmp (*iter, LOGD_DEFAULT_STRING)) {
 			bits = LOGD_DEFAULT;
-		else if (!g_ascii_strcasecmp (*iter, LOGD_DHCP_STRING))
+			protect = LOGD_VPN_PLUGIN;
+		} else if (!g_ascii_strcasecmp (*iter, LOGD_DHCP_STRING))
 			bits = LOGD_DHCP;
 		else if (!g_ascii_strcasecmp (*iter, LOGD_IP_STRING))
 			bits = LOGD_IP;
@@ -321,13 +380,17 @@ nm_logging_setup (const char  *level,
 
 		if (domain_log_level == _LOGL_KEEP) {
 			for (i = 0; i < G_N_ELEMENTS (new_logging); i++)
-				new_logging[i] = (new_logging[i] & ~bits) | (global.logging[i] & bits);
+				new_logging[i] = (new_logging[i] & ~bits) | (_nm_logging_enabled_state[i] & bits);
 		} else {
 			for (i = 0; i < G_N_ELEMENTS (new_logging); i++) {
 				if (i < domain_log_level)
 					new_logging[i] &= ~bits;
-				else
+				else {
 					new_logging[i] |= bits;
+					if (   (protect & bits)
+					    && i < LOGL_INFO)
+						new_logging[i] &= ~protect;
+				}
 			}
 		}
 	}
@@ -339,7 +402,7 @@ nm_logging_setup (const char  *level,
 
 	global.log_level = new_log_level;
 	for (i = 0; i < G_N_ELEMENTS (new_logging); i++)
-		global.logging[i] = new_logging[i];
+		_nm_logging_enabled_state[i] = new_logging[i];
 
 	if (   had_platform_debug
 	    && _nm_logging_clear_platform_logging_cache
@@ -384,8 +447,6 @@ nm_logging_all_levels_to_string (void)
 const char *
 nm_logging_domains_to_string (void)
 {
-	_ensure_initialized ();
-
 	if (G_UNLIKELY (!global.logging_domains_to_string))
 		global.logging_domains_to_string = _domains_to_string (TRUE);
 
@@ -406,7 +467,7 @@ _domains_to_string (gboolean include_level_override)
 	str = g_string_sized_new (75);
 	for (diter = &global.domain_desc[0]; diter->name; diter++) {
 		/* If it's set for any lower level, it will also be set for LOGL_ERR */
-		if (!(diter->num & global.logging[LOGL_ERR]))
+		if (!(diter->num & _nm_logging_enabled_state[LOGL_ERR]))
 			continue;
 
 		if (str->len)
@@ -418,15 +479,15 @@ _domains_to_string (gboolean include_level_override)
 
 		/* Check if it's logging at a lower level than the default. */
 		for (i = 0; i < global.log_level; i++) {
-			if (diter->num & global.logging[i]) {
+			if (diter->num & _nm_logging_enabled_state[i]) {
 				g_string_append_printf (str, ":%s", global.level_desc[i].name);
 				break;
 			}
 		}
 		/* Check if it's logging at a higher level than the default. */
-		if (!(diter->num & global.logging[global.log_level])) {
-			for (i = global.log_level + 1; i < G_N_ELEMENTS (global.logging); i++) {
-				if (diter->num & global.logging[i]) {
+		if (!(diter->num & _nm_logging_enabled_state[global.log_level])) {
+			for (i = global.log_level + 1; i < G_N_ELEMENTS (_nm_logging_enabled_state); i++) {
+				if (diter->num & _nm_logging_enabled_state[i]) {
 					g_string_append_printf (str, ":%s", global.level_desc[i].name);
 					break;
 				}
@@ -459,22 +520,44 @@ nm_logging_all_domains_to_string (void)
 	return str->str;
 }
 
-gboolean
-nm_logging_enabled (NMLogLevel level, NMLogDomain domain)
+/**
+ * nm_logging_get_level:
+ * @domain: find the lowest enabled logging level for the
+ *   given domain. If this is a set of multiple
+ *   domains, the most verbose level will be returned.
+ *
+ * Returns: the lowest (most verbose) logging level for the
+ *   give @domain, or %_LOGL_OFF if it is disabled.
+ **/
+NMLogLevel
+nm_logging_get_level (NMLogDomain domain)
 {
-	if ((guint) level >= G_N_ELEMENTS (global.logging))
-		g_return_val_if_reached (FALSE);
+	NMLogLevel sl = _LOGL_OFF;
 
-	/* This function is guaranteed not to modify errno. */
-	_ensure_initialized ();
-
-	return !!(global.logging[level] & domain);
+	G_STATIC_ASSERT (LOGL_TRACE == 0);
+	while (   sl > LOGL_TRACE
+	       && nm_logging_enabled (sl - 1, domain))
+		sl--;
+	return sl;
 }
 
 #if SYSTEMD_JOURNAL
-__attribute__((__format__ (__printf__, 4, 5)))
 static void
-_iovec_set_format (struct iovec *iov, gboolean *iov_free, int i, const char *format, ...)
+_iovec_set (struct iovec *iov, const void *str, gsize len)
+{
+	iov->iov_base = (void *) str;
+	iov->iov_len = len;
+}
+
+static void
+_iovec_set_string (struct iovec *iov, const char *str)
+{
+	_iovec_set (iov, str, strlen (str));
+}
+
+_nm_printf (3, 4)
+static void
+_iovec_set_format (struct iovec *iov, gpointer *iov_free, const char *format, ...)
 {
 	va_list ap;
 	char *str;
@@ -483,19 +566,24 @@ _iovec_set_format (struct iovec *iov, gboolean *iov_free, int i, const char *for
 	str = g_strdup_vprintf (format, ap);
 	va_end (ap);
 
-	iov[i].iov_base = str;
-	iov[i].iov_len = strlen (str);
-	iov_free[i] = TRUE;
+	_iovec_set_string (iov, str);
+	*iov_free = str;
 }
 
-static void
-_iovec_set_string (struct iovec *iov, gboolean *iov_free, int i, const char *str, gsize len)
-{
-	iov[i].iov_base = (char *) str;
-	iov[i].iov_len = len;
-	iov_free[i] = FALSE;
-}
-#define _iovec_set_literal_string(iov, iov_free, i, str) _iovec_set_string ((iov), (iov_free), (i), (""str""), NM_STRLEN (str))
+#define _iovec_set_format_a(iov, reserve_extra, format, ...) \
+	G_STMT_START { \
+		const gsize _size = (reserve_extra) + (NM_STRLEN (format) + 3); \
+		char *const _buf = g_alloca (_size); \
+		int _len; \
+		\
+		_len = g_snprintf (_buf, _size, ""format"", ##__VA_ARGS__);\
+		\
+		nm_assert (_len >= 0); \
+		nm_assert (_len <= _size); \
+		nm_assert (_len == strlen (_buf)); \
+		\
+		_iovec_set ((iov), _buf, _len); \
+	} G_STMT_END
 #endif
 
 void
@@ -510,19 +598,17 @@ _nm_log_impl (const char *file,
 {
 	va_list args;
 	char *msg;
-	char *fullmsg;
-	char s_buf_timestamp[64];
-	char s_buf_location[1024];
 	GTimeVal tv;
+	int errno_saved;
 
-	if ((guint) level >= G_N_ELEMENTS (global.logging))
+	if ((guint) level >= G_N_ELEMENTS (_nm_logging_enabled_state))
 		g_return_if_reached ();
 
-	_ensure_initialized ();
-
-	if (!(global.logging[level] & domain))
+	if (!(_nm_logging_enabled_state[level] & domain))
 		return;
 
+	errno_saved = errno;
+
 	/* Make sure that %m maps to the specified error */
 	if (error != 0) {
 		if (error < 0)
@@ -534,51 +620,15 @@ _nm_log_impl (const char *file,
 	msg = g_strdup_vprintf (fmt, args);
 	va_end (args);
 
-	if (NM_FLAGS_ANY (global.log_format_flags, global.level_desc[level].log_format_level & _LOG_FORMAT_FLAG_TIMESTAMP)) {
-		g_get_current_time (&tv);
-		nm_sprintf_buf (s_buf_timestamp, " [%ld.%04ld]", tv.tv_sec, (tv.tv_usec + 50) / 100);
-	} else
-		s_buf_timestamp[0] = '\0';
+#define MESSAGE_FMT "%s%-7s [%ld.%04ld] %s"
+#define MESSAGE_ARG(global, tv, msg) \
+    (global).prefix, \
+    (global).level_desc[level].level_str, \
+    (tv).tv_sec, \
+    ((tv).tv_usec / 100), \
+    (msg)
 
-	s_buf_location[0] = '\0';
-	if (NM_FLAGS_ANY (global.log_format_flags, global.level_desc[level].log_format_level & _LOG_FORMAT_FLAG_LOCATION)) {
-#define MAX_LEN_FILE 37
-#define MAX_LEN_FUNC 26
-		gsize l = sizeof (s_buf_location);
-		char *p = s_buf_location, *p_buf;
-		gsize len;
-		char s_buf[MAX (MAX_LEN_FILE, MAX_LEN_FUNC) + 30];
-
-		if (file) {
-			if (NM_FLAGS_HAS (global.log_format_flags, LOG_FORMAT_FLAG_ALIGN_LOCATION)) {
-				/* left-align the "[file:line]" string, but truncate from left to MAX_LEN_FILE chars. */
-				len = strlen (file);
-				nm_sprintf_buf (s_buf, "[%s:%u]",
-				                len > MAX_LEN_FILE ? &file[len - MAX_LEN_FILE] : file,
-				                line);
-				len = strlen (s_buf);
-				if (len > MAX_LEN_FILE) {
-					p_buf = &s_buf[len - MAX_LEN_FILE];
-					p_buf[0] = '[';
-				} else
-					p_buf = s_buf;
-				nm_utils_strbuf_append (&p, &l, " %-"G_STRINGIFY (MAX_LEN_FILE)"s", p_buf);
-			} else
-				nm_utils_strbuf_append (&p, &l, " [%s:%u]", file, line);
-		}
-		if (func) {
-			if (NM_FLAGS_HAS (global.log_format_flags, LOG_FORMAT_FLAG_ALIGN_LOCATION)) {
-				/* left-align the "func():" string, but truncate from left to MAX_LEN_FUNC chars. */
-				len = strlen (func);
-				nm_sprintf_buf (s_buf, "%s():",
-				                len > MAX_LEN_FUNC ? &func[len - MAX_LEN_FUNC] : func);
-				len = strlen (s_buf);
-				nm_utils_strbuf_append (&p, &l, " %-"G_STRINGIFY (MAX_LEN_FUNC)"s",
-				                        len > MAX_LEN_FUNC ? &s_buf[len - MAX_LEN_FUNC] : s_buf);
-			} else
-				nm_utils_strbuf_append (&p, &l, " %s():", func);
-		}
-	}
+	g_get_current_time (&tv);
 
 	switch (global.log_backend) {
 #if SYSTEMD_JOURNAL
@@ -586,30 +636,25 @@ _nm_log_impl (const char *file,
 		{
 			gint64 now, boottime;
 #define _NUM_MAX_FIELDS_SYSLOG_FACILITY 10
-#define _NUM_FIELDS (10 + _NUM_MAX_FIELDS_SYSLOG_FACILITY)
-			int i_field = 0;
-			struct iovec iov[_NUM_FIELDS];
-			gboolean iov_free[_NUM_FIELDS];
+			struct iovec iov_data[12 + _NUM_MAX_FIELDS_SYSLOG_FACILITY];
+			struct iovec *iov = iov_data;
+			gpointer iov_free_data[3];
+			gpointer *iov_free = iov_free_data;
+			nm_auto_free_gstring GString *s_domain_all = NULL;
 
 			now = nm_utils_get_monotonic_timestamp_ns ();
 			boottime = nm_utils_monotonic_timestamp_as_boottime (now, 1);
 
-			_iovec_set_format (iov, iov_free, i_field++, "PRIORITY=%d", global.level_desc[level].syslog_level);
-			_iovec_set_format (iov, iov_free, i_field++, "MESSAGE="
-			                   "%-7s%s%s %s",
-			                   global.level_desc[level].level_str,
-			                   s_buf_timestamp,
-			                   s_buf_location,
-			                   msg);
-			_iovec_set_literal_string (iov, iov_free, i_field++, "SYSLOG_IDENTIFIER=" G_LOG_DOMAIN);
-			_iovec_set_format (iov, iov_free, i_field++, "SYSLOG_PID=%ld", (long) getpid ());
+			_iovec_set_format_a (iov++, 30, "PRIORITY=%d", global.level_desc[level].syslog_level);
+			_iovec_set_format (iov++, iov_free++, "MESSAGE="MESSAGE_FMT, MESSAGE_ARG (global, tv, msg));
+			_iovec_set_string (iov++, syslog_identifier_full (&global));
+			_iovec_set_format_a (iov++, 30, "SYSLOG_PID=%ld", (long) getpid ());
 			{
 				const LogDesc *diter;
 				int i_domain = _NUM_MAX_FIELDS_SYSLOG_FACILITY;
 				const char *s_domain_1 = NULL;
-				GString *s_domain_all = NULL;
 				NMLogDomain dom_all = domain;
-				NMLogDomain dom = dom_all & global.logging[level];
+				NMLogDomain dom = dom_all & _nm_logging_enabled_state[level];
 
 				for (diter = &global.domain_desc[0]; diter->name; diter++) {
 					if (!NM_FLAGS_HAS (dom_all, diter->num))
@@ -622,8 +667,10 @@ _nm_log_impl (const char *file,
 					if (!s_domain_1)
 						s_domain_1 = diter->name;
 					else {
-						if (!s_domain_all)
-							s_domain_all = g_string_new (s_domain_1);
+						if (!s_domain_all) {
+							s_domain_all = g_string_new ("NM_LOG_DOMAINS=");
+							g_string_append (s_domain_all, s_domain_1);
+						}
 						g_string_append_c (s_domain_all, ',');
 						g_string_append (s_domain_all, diter->name);
 					}
@@ -631,7 +678,7 @@ _nm_log_impl (const char *file,
 					if (NM_FLAGS_HAS (dom, diter->num)) {
 						if (i_domain > 0) {
 							/* SYSLOG_FACILITY is specified multiple times for each domain that is actually enabled. */
-							_iovec_set_format (iov, iov_free, i_field++, "SYSLOG_FACILITY=%s", diter->name);
+							_iovec_set_format_a (iov++, _MAX_LEN (30, diter->name), "SYSLOG_FACILITY=%s", diter->name);
 							i_domain--;
 						}
 						dom &= ~diter->num;
@@ -639,52 +686,47 @@ _nm_log_impl (const char *file,
 					if (!dom && !dom_all)
 						break;
 				}
-				if (s_domain_all) {
-					_iovec_set_format (iov, iov_free, i_field++, "NM_LOG_DOMAINS=%s", s_domain_all->str);
-					g_string_free (s_domain_all, TRUE);
-				} else
-					_iovec_set_format (iov, iov_free, i_field++, "NM_LOG_DOMAINS=%s", s_domain_1);
+				if (s_domain_all)
+					_iovec_set (iov++, s_domain_all->str, s_domain_all->len);
+				else
+					_iovec_set_format_a (iov++, _MAX_LEN (30, s_domain_1), "NM_LOG_DOMAINS=%s", s_domain_1);
 			}
-			_iovec_set_format (iov, iov_free, i_field++, "NM_LOG_LEVEL=%s", global.level_desc[level].name);
-			_iovec_set_format (iov, iov_free, i_field++, "CODE_FUNC=%s", func ?: "");
-			_iovec_set_format (iov, iov_free, i_field++, "CODE_FILE=%s", file ?: "");
-			_iovec_set_format (iov, iov_free, i_field++, "CODE_LINE=%u", line);
-			_iovec_set_format (iov, iov_free, i_field++, "TIMESTAMP_MONOTONIC=%lld.%06lld", (long long) (now / NM_UTILS_NS_PER_SECOND), (long long) ((now % NM_UTILS_NS_PER_SECOND) / 1000));
-			_iovec_set_format (iov, iov_free, i_field++, "TIMESTAMP_BOOTTIME=%lld.%06lld", (long long) (boottime / NM_UTILS_NS_PER_SECOND), (long long) ((boottime % NM_UTILS_NS_PER_SECOND) / 1000));
+			_iovec_set_format_a (iov++, _MAX_LEN (15, global.level_desc[level].name), "NM_LOG_LEVEL=%s", global.level_desc[level].name);
+			if (func)
+				_iovec_set_format (iov++, iov_free++, "CODE_FUNC=%s", func);
+			_iovec_set_format (iov++, iov_free++, "CODE_FILE=%s", file ?: "");
+			_iovec_set_format_a (iov++, 20, "CODE_LINE=%u", line);
+			_iovec_set_format_a (iov++, 60, "TIMESTAMP_MONOTONIC=%lld.%06lld", (long long) (now / NM_UTILS_NS_PER_SECOND), (long long) ((now % NM_UTILS_NS_PER_SECOND) / 1000));
+			_iovec_set_format_a (iov++, 60, "TIMESTAMP_BOOTTIME=%lld.%06lld", (long long) (boottime / NM_UTILS_NS_PER_SECOND), (long long) ((boottime % NM_UTILS_NS_PER_SECOND) / 1000));
 			if (error != 0)
-				_iovec_set_format (iov, iov_free, i_field++, "ERRNO=%d", error);
+				_iovec_set_format_a (iov++, 30, "ERRNO=%d", error);
 
-			nm_assert (i_field <= G_N_ELEMENTS (iov));
+			nm_assert (iov <= &iov_data[G_N_ELEMENTS (iov_data)]);
+			nm_assert (iov_free <= &iov_free_data[G_N_ELEMENTS (iov_free_data)]);
 
-			sd_journal_sendv (iov, i_field);
+			sd_journal_sendv (iov_data, iov - iov_data);
 
-			for (; i_field > 0; ) {
-				i_field--;
-				if (iov_free[i_field])
-					g_free (iov[i_field].iov_base);
-			}
+			for (; --iov_free >= iov_free_data; )
+				g_free (*iov_free);
 		}
 		break;
 #endif
+	case LOG_BACKEND_SYSLOG:
+		syslog (global.level_desc[level].syslog_level,
+		        MESSAGE_FMT, MESSAGE_ARG (global, tv, msg));
+		break;
 	default:
-		fullmsg = g_strdup_printf ("%-7s%s%s %s",
-		                           global.level_desc[level].level_str,
-		                           s_buf_timestamp,
-		                           s_buf_location,
-		                           msg);
-
-		if (global.log_backend == LOG_BACKEND_SYSLOG)
-			syslog (global.level_desc[level].syslog_level, "%s", fullmsg);
-		else
-			g_log (G_LOG_DOMAIN, global.level_desc[level].g_log_level, "%s", fullmsg);
-		g_free (fullmsg);
+		g_log (syslog_identifier_domain (&global), global.level_desc[level].g_log_level,
+		       MESSAGE_FMT, MESSAGE_ARG (global, tv, msg));
 		break;
 	}
 
 	g_free (msg);
+
+	errno = errno_saved;
 }
 
-/************************************************************************/
+/*****************************************************************************/
 
 static void
 nm_log_handler (const gchar *log_domain,
@@ -726,8 +768,8 @@ nm_log_handler (const gchar *log_domain,
 			boottime = nm_utils_monotonic_timestamp_as_boottime (now, 1);
 
 			sd_journal_send ("PRIORITY=%d", syslog_priority,
-			                 "MESSAGE=%s", message ?: "",
-			                 "SYSLOG_IDENTIFIER=%s", G_LOG_DOMAIN,
+			                 "MESSAGE=%s%s", global.prefix, message ?: "",
+			                 syslog_identifier_full (&global),
 			                 "SYSLOG_PID=%ld", (long) getpid (),
 			                 "SYSLOG_FACILITY=GLIB",
 			                 "GLIB_DOMAIN=%s", log_domain ?: "",
@@ -739,30 +781,57 @@ nm_log_handler (const gchar *log_domain,
 		break;
 #endif
 	default:
-		syslog (syslog_priority, "%s", message ?: "");
+		syslog (syslog_priority, "%s%s", global.prefix, message ?: "");
 		break;
 	}
 }
 
+gboolean
+nm_logging_syslog_enabled (void)
+{
+	return global.uses_syslog;
+}
+
+void
+nm_logging_set_prefix (const char *format, ...)
+{
+	char *prefix;
+	va_list ap;
+
+	/* prefix can only be set once, to a non-empty string. Also, after
+	 * nm_logging_syslog_openlog() the prefix cannot be set either. */
+	if (global.log_backend != LOG_BACKEND_GLIB)
+		g_return_if_reached ();
+	if (global.prefix[0])
+		g_return_if_reached ();
+
+	va_start (ap, format);
+	prefix = g_strdup_vprintf (format, ap);
+	va_end (ap);
+
+	if (!prefix || !prefix[0])
+		g_return_if_reached ();
+
+	/* we pass the allocated string on and never free it. */
+	global.prefix = prefix;
+}
+
 void
 nm_logging_syslog_openlog (const char *logging_backend)
 {
-	LogFormatFlags log_format_flags;
-
 	if (global.log_backend != LOG_BACKEND_GLIB)
 		g_return_if_reached ();
 
 	if (!logging_backend)
-		logging_backend = ""NM_CONFIG_LOGGING_BACKEND_DEFAULT;
-
-	log_format_flags = _LOG_FORMAT_FLAG_DEFAULT;
+		logging_backend = ""NM_CONFIG_DEFAULT_LOGGING_BACKEND;
 
 	if (strcmp (logging_backend, "debug") == 0) {
 		global.log_backend = LOG_BACKEND_SYSLOG;
-		openlog (G_LOG_DOMAIN, LOG_CONS | LOG_PERROR | LOG_PID, LOG_USER);
+		openlog (syslog_identifier_domain (&global), LOG_CONS | LOG_PERROR | LOG_PID, LOG_USER);
 #if SYSTEMD_JOURNAL
 	} else if (strcmp (logging_backend, "syslog") != 0) {
 		global.log_backend = LOG_BACKEND_JOURNAL;
+		global.uses_syslog = TRUE;
 
 		/* ensure we read a monotonic timestamp. Reading the timestamp the first
 		 * time causes a logging message. We don't want to do that during _nm_log_impl. */
@@ -770,12 +839,11 @@ nm_logging_syslog_openlog (const char *logging_backend)
 #endif
 	} else {
 		global.log_backend = LOG_BACKEND_SYSLOG;
-		openlog (G_LOG_DOMAIN, LOG_PID, LOG_DAEMON);
+		global.uses_syslog = TRUE;
+		openlog (syslog_identifier_domain (&global), LOG_PID, LOG_DAEMON);
 	}
 
-	global.log_format_flags = log_format_flags;
-
-	g_log_set_handler (G_LOG_DOMAIN,
+	g_log_set_handler (syslog_identifier_domain (&global),
 	                   G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
 	                   nm_log_handler,
 	                   NULL);
