@@ -31,19 +31,12 @@
 #ifndef _SHVAR_H
 #define _SHVAR_H
 
-#include "nm-default.h"
-
-G_BEGIN_DECLS
-
 typedef struct _shvarFile shvarFile;
-struct _shvarFile {
-	char      *fileName;    /* read-only */
-	int        fd;          /* read-only */
-	GList     *lineList;    /* read-only */
-	GList     *current;     /* set implicitly or explicitly, points to element of lineList */
-	gboolean   modified;    /* ignore */
-};
 
+const char *svFileGetName (const shvarFile *s);
+void svFileSetName (shvarFile *s, const char *fileName);
+
+void svFileSetModified (shvarFile *s);
 
 /* Create the file <name>, return a shvarFile (never fails) */
 shvarFile *svCreateFile (const char *name);
@@ -55,8 +48,8 @@ shvarFile *svOpenFile (const char *name, GError **error);
  * pointing at the line containing the value.  The char* returned MUST
  * be freed by the caller.
  */
-char *svGetValue (shvarFile *s, const char *key, gboolean verbatim);
-char *svGetValueFull (shvarFile *s, const char *key, gboolean verbatim);
+const char *svGetValue (shvarFile *s, const char *key, char **to_free);
+char *svGetValueString (shvarFile *s, const char *key);
 
 gint svParseBoolean (const char *value, gint def);
 
@@ -73,10 +66,12 @@ gint64 svGetValueInt64 (shvarFile *s, const char *key, guint base, gint64 min, g
  * the key=value pair after that line.  Otherwise, prepend the pair
  * to the top of the file.
  */
-void svSetValue (shvarFile *s, const char *key, const char *value, gboolean verbatim);
-void svSetValueFull (shvarFile *s, const char *key, const char *value, gboolean verbatim);
+void svSetValueString (shvarFile *s, const char *key, const char *value);
+void svSetValue (shvarFile *s, const char *key, const char *value);
+void svSetValueBoolean (shvarFile *s, const char *key, gboolean value);
 void svSetValueInt64 (shvarFile *s, const char *key, gint64 value);
 
+void svUnsetValue (shvarFile *s, const char *key);
 
 /* Write the current contents iff modified.  Returns FALSE on error
  * and TRUE on success.  Do not write if no values have been modified.
@@ -89,12 +84,7 @@ gboolean svWriteFile (shvarFile *s, int mode, GError **error);
 /* Close the file descriptor (if open) and free the shvarFile. */
 void svCloseFile (shvarFile *s);
 
-/* Return @s unmodified or an escaped string */
 const char *svEscape (const char *s, char **to_free);
+const char *svUnescape (const char *s, char **to_free);
 
-/* Unescape a string in-place */
-void svUnescape (char *s);
-
-G_END_DECLS
-
-#endif /* ! _SHVAR_H */
+#endif /* _SHVAR_H */
